/// <reference types="node" />
/// <reference types="ioredis" />
import * as Redis from 'ioredis';
import * as event from 'events';
/**
 * @callback MessageHandler
 * @param message {String}
 * @returns {Promise}
 */
export interface MessageHandler {
    (message: string): Promise<any>;
}
export declare class RedisQueueWatchdog extends event.EventEmitter {
    private table;
    private watchdogTopic;
    private watchdogTimeout;
    private setTimeout;
    private clearTimeout;
    private redis;
    private watchdogRedis;
    private topicListener;
    private state;
    /**
     * Creating a watchdog instance
     *
     * @param opt.watchdogRedis Instance of IORedis that will enter subscribe mode, can pass null and then create it internally
     * @param opt.watchdogRedisHost {String} Host for internal watchdog IORedis instance
     * @param opt.watchdogRedisPort {Number} Port for internal watchdog IORedis instance
     * @param opt.watchdogRedisSpace {Number} Database index for internal watchdog IORedis instance
     * @param opt.redis Instance of IORedis that will manipulate the queue, can pass null and then create it internally.
     * @param opt.redisHost {String} Host for internal IORedis instance
     * @param opt.redisPort {Number} Port for internal IORedis instance
     * @param opt.redisSpace {Number} Database index for internal IORedis instance
     * @param opt.watchdogTimeout {Number} Timeout of the watchdog
     * @param opt.watchdogTopic {Number} Topic name which watchdog is subscribe to
     * @param opt.setTimeout {Function} setTimeout function, pass a stub if you want to do test.
     * @param opt.clearTimeout {Function} clearTimeout function, pass a stub if you want to do test.
     */
    constructor(opt: RedisQueueWatchdog.Config);
    /**
     * Start the watchdog
     * @returns {Promise}
     */
    start(): Promise<void>;
    /**
     * Stop the watchdog
     * @returns {Promise}
     */
    stop(): Promise<{}>;
}
export declare namespace RedisQueueWatchdog {
    interface Config {
        watchdogTopic: string;
        watchdogTimeout?: number;
        setTimeout?: typeof global.setTimeout;
        clearTimeout?: typeof global.clearTimeout;
        redisHost?: string;
        redisPort?: number;
        redisSpace?: number;
        watchdogRedisHost?: string;
        watchdogRedisPort?: number;
        watchdogRedisSpace?: number;
        redis?: Redis.Redis;
        watchdogRedis?: Redis.Redis;
    }
}
export declare class RedisQueueProducer extends event.EventEmitter {
    private redis;
    private queue;
    /**
     * Creating a watchdog instance
     *
     * @param option.redis Instance of IORedis that will manipulate the queue, can pass null and then create it internally.
     * @param option.redisHost {String} Host for internal IORedis instance
     * @param option.redisPort {Number} Port for internal IORedis instance
     * @param option.redisSpace {Number} Database index for internal IORedis instance
     * @param option.queue {String} The name of the queue in redis
     */
    constructor(option: RedisQueueProducer.Config);
    /**
     * Send message
     * @param message The message
     * @returns {Promise}
     */
    send(message: any): Promise<any>;
}
export declare namespace RedisQueueProducer {
    interface Config {
        redis?: Redis.Redis;
        redisHost?: string;
        redisPort?: number;
        redisSpace?: number;
        queue: string;
    }
}
export declare class RedisQueueConsumer extends event.EventEmitter {
    private watchdogTopic;
    private watchdogTimeout?;
    private redis;
    private watchdogRedis;
    private queue;
    private state;
    private idGenerator;
    /**
     * Creating a watchdog instance
     *
     * @param option.watchdogRedis Instance of IORedis that will enter subscribe mode, can pass null and then create it internally
     * @param option.watchdogRedisHost {String} Host for internal watchdog IORedis instance
     * @param option.watchdogRedisPort {Number} Port for internal watchdog IORedis instance
     * @param option.watchdogRedisSpace {Number} Database index for internal watchdog IORedis instance
     * @param option.redis Instance of IORedis that will manipulate the queue, can pass null and then create it internally.
     * @param option.redisHost {String} Host for internal IORedis instance
     * @param option.redisPort {Number} Port for internal IORedis instance
     * @param option.redisSpace {Number} Database index for internal IORedis instance
     * @param option.watchdogTimeout {Number} Timeout of the watchdog
     * @param option.watchdogTopic {Number} Topic name which watchdog is subscribe to
     * @param option.queue {String} The name of the queue in redis
     * @param option.setInterval {Function} setInterval function, pass a stub if you want to do test.
     * @param option.clearInterval {Function} clearInterval function, pass a stub if you want to do test.
     */
    constructor(option: RedisQueueConsumer.Config);
    private _heartbeat(sponge);
    /**
     * Start queue consumer
     * @param consumer {MessageHandler} function handle queue element
     */
    start(consumer: MessageHandler): Promise<void>;
    /**
     * Stop queue consumer
     */
    stop(): Promise<void>;
}
export declare namespace RedisQueueConsumer {
    interface Config {
        watchdogTopic: string;
        watchdogTimeout?: number;
        queue: string;
        redisHost?: string;
        redisPort?: number;
        redisSpace?: number;
        watchdogRedisHost?: string;
        watchdogRedisPort?: number;
        watchdogRedisSpace?: number;
        redis?: Redis.Redis;
        watchdogRedis?: Redis.Redis;
    }
}
