"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const Redis = require('ioredis');
const assert = require('assert');
const tuid = require('tuid');
const index_1 = require('../index');
const TEST_QUEUE = 'test-queue';
const TEST_SPONGE = 'test-sponge';
const TEST_TOPIC = 'test-watch-dog';
const consumerOpt = { watchdogTopic: TEST_TOPIC, queue: TEST_QUEUE, watchdogTimeout: 1000 };
describe('RedisQueueWatchdog', function () {
    "use strict";
    this.slow(3000).timeout(5000);
    it('should recover elements from sponge to queue', () => __awaiter(this, void 0, void 0, function* () {
        let redis = new Redis();
        let timeout = false;
        let x = new index_1.RedisQueueWatchdog({
            watchdogTopic: TEST_TOPIC,
            setTimeout: (f) => setImmediate(() => { timeout = true; f(); }),
            clearTimeout: (f) => clearImmediate(f)
        });
        yield x.start();
        yield redis.del(TEST_QUEUE);
        yield redis.del(TEST_SPONGE);
        yield redis.lpush(TEST_QUEUE, '1');
        yield redis.rpoplpush(TEST_QUEUE, TEST_SPONGE);
        assert.equal(yield redis.llen(TEST_QUEUE), 0);
        assert.equal(yield redis.llen(TEST_SPONGE), 1);
        yield redis.publish(TEST_TOPIC, JSON.stringify({ sponge: TEST_SPONGE, queue: TEST_QUEUE }));
        yield new Promise(done => setTimeout(done, 100));
        yield x.stop();
        assert(timeout);
        assert.notEqual(yield redis.llen(TEST_QUEUE), 0);
        assert.equal(yield redis.llen(TEST_SPONGE), 0);
        yield redis.quit();
    }));
    it('should should call clearTimeout if heartbeat continues', () => __awaiter(this, void 0, void 0, function* () {
        let redis = new Redis();
        let cancelled = false;
        let x = new index_1.RedisQueueWatchdog({
            watchdogTopic: TEST_TOPIC,
            setTimeout: setTimeout,
            clearTimeout: (f) => { cancelled = true; clearTimeout(f); }
        });
        yield x.start();
        yield redis.del(TEST_QUEUE);
        yield redis.del(TEST_SPONGE);
        yield redis.lpush(TEST_QUEUE, '1');
        yield redis.rpoplpush(TEST_QUEUE, TEST_SPONGE);
        yield redis.publish(TEST_TOPIC, JSON.stringify({ sponge: TEST_SPONGE, queue: TEST_QUEUE }));
        assert.equal(cancelled, false);
        yield new Promise(done => setTimeout(done, 100));
        yield redis.publish(TEST_TOPIC, JSON.stringify({ sponge: TEST_SPONGE, queue: TEST_QUEUE }));
        yield new Promise(done => setTimeout(done, 100));
        assert.equal(cancelled, true);
        yield x.stop();
        yield redis.quit();
    }));
    it('should start/stop normally in any state', () => __awaiter(this, void 0, void 0, function* () {
        let watchdog = new index_1.RedisQueueWatchdog(consumerOpt);
        watchdog.start();
        yield watchdog.start();
        yield watchdog.start();
        watchdog.start();
        watchdog.stop();
        watchdog.stop();
        yield watchdog.stop();
        yield watchdog.stop();
        watchdog.stop();
    }));
});
describe('Consumer', function () {
    this.slow(3000).timeout(5000);
    it('should be able to start and stop', function () {
        return __awaiter(this, void 0, void 0, function* () {
            let consumer = new index_1.RedisQueueConsumer(consumerOpt);
            yield consumer.start(() => __awaiter(this, void 0, void 0, function* () { return null; }));
            yield new Promise(done => setTimeout(done, 100));
            yield consumer.stop();
        });
    });
    it('should heartbeats', function () {
        return __awaiter(this, void 0, void 0, function* () {
            var exception = null;
            let consumer = new index_1.RedisQueueConsumer(consumerOpt);
            consumer.on('error', (e) => { console.error(e); exception = e; });
            var redis = new Redis();
            yield redis.subscribe(TEST_TOPIC);
            yield consumer.start(() => __awaiter(this, void 0, void 0, function* () { return undefined; }));
            let counter = 0;
            yield new Promise((done) => redis.on('message', (topic) => topic == TEST_TOPIC && ++counter == 2 && done()));
            yield consumer.stop();
            yield redis.unsubscribe(TEST_TOPIC);
            yield redis.quit();
            assert(exception == null);
        });
    });
    it('should start/stop normally in any state', function () {
        return __awaiter(this, void 0, void 0, function* () {
            let consumer = new index_1.RedisQueueConsumer(consumerOpt);
            consumer.start(() => __awaiter(this, void 0, void 0, function* () { return undefined; }));
            yield consumer.start(() => __awaiter(this, void 0, void 0, function* () { return undefined; }));
            yield consumer.start(() => __awaiter(this, void 0, void 0, function* () { return undefined; }));
            consumer.start(() => __awaiter(this, void 0, void 0, function* () { return undefined; }));
            consumer.stop();
            consumer.stop();
            yield consumer.stop();
            yield consumer.stop();
            consumer.stop();
        });
    });
});
describe('Consumer and Producer', function () {
    this.slow(5000).timeout(10000);
    it('should receive message', () => __awaiter(this, void 0, void 0, function* () {
        "use strict";
        let consumer = new index_1.RedisQueueConsumer(consumerOpt);
        let received = false;
        yield consumer.start(() => __awaiter(this, void 0, void 0, function* () { return received = true; }));
        let producer = new index_1.RedisQueueProducer({ queue: TEST_QUEUE });
        yield producer.send('test');
        yield new Promise(done => setTimeout(done, 100));
        yield consumer.stop();
        assert(received == true);
    }));
    it('should emit error event when handler throw exception', () => __awaiter(this, void 0, void 0, function* () {
        let consumer = new index_1.RedisQueueConsumer(consumerOpt);
        let exception = null;
        consumer.on('error', (e) => {
            exception = e;
        });
        yield consumer.start((message) => __awaiter(this, void 0, void 0, function* () { throw new Error(`Receive message: ${message}`); }));
        let producer = new index_1.RedisQueueProducer({ queue: TEST_QUEUE });
        yield producer.send('test');
        yield new Promise(done => setTimeout(done, 100));
        yield consumer.stop();
        assert(exception != null);
    }));
    it('should continue receive message after error occurred', () => __awaiter(this, void 0, void 0, function* () {
        let watchdog = new index_1.RedisQueueWatchdog(consumerOpt);
        yield watchdog.start();
        let consumer = new index_1.RedisQueueConsumer(consumerOpt);
        consumer.once('error', (e) => {
        });
        let state = 0;
        yield Promise.all([new Promise((done) => {
                consumer.start((message) => __awaiter(this, void 0, void 0, function* () {
                    switch (state) {
                        case 0:
                            state++;
                            assert(message == 'failed');
                            throw new Error(`Receive message: ${message}`);
                        case 1:
                            state++;
                            assert(message == 'success');
                            return;
                        case 2:
                            assert(message == 'failed');
                            done();
                            return;
                    }
                }));
            }), (() => __awaiter(this, void 0, void 0, function* () {
                let producer = new index_1.RedisQueueProducer({ queue: TEST_QUEUE });
                yield producer.send('failed');
                yield producer.send('success');
            }))()]);
        yield consumer.stop();
        yield watchdog.stop();
    }));
    it('should not receive message exactly as passed', () => __awaiter(this, void 0, void 0, function* () {
        "use strict";
        let values = [];
        for (let i = 0; i < 100; i++)
            values.push(i);
        let consumers = values.map(x => new index_1.RedisQueueConsumer(consumerOpt));
        let receivedMessage = [];
        let sentMessage = [];
        const MAX_MESSAGE_COUNT = 5000;
        let receivedCount = 0;
        yield new Promise((done) => {
            consumers.forEach(consumer => {
                return consumer.start((message) => __awaiter(this, void 0, void 0, function* () {
                    receivedMessage.push(message);
                    receivedCount++;
                    if (receivedCount == MAX_MESSAGE_COUNT)
                        done();
                }));
            });
            let producers = values.map(x => new index_1.RedisQueueProducer({ queue: TEST_QUEUE }));
            for (let i = 0; i < MAX_MESSAGE_COUNT; i++) {
                let message = new tuid.Generator().generateSync().toString();
                sentMessage.push(message);
                producers[Date.now() % 100].send(message);
            }
        });
        assert.deepEqual(sentMessage.sort(), receivedMessage.sort());
        yield Promise.all(consumers.map(c => c.stop));
    }));
});
//# sourceMappingURL=test.js.map